# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from abc import abstractmethod
from bayeslibs.motion import robot_move_forward, get_robot_move_status, robot_move_back
from bayeslibs.motion import get_robot_rotate_status, robot_rotate_left, robot_rotate_right
from bayeslibs.voice import start_music, stop_music, get_music_status
import time


class ApolloMusicPlayer:
    def __init__(self):
        self.__music_name = None

    def get_music_name(self):
        return self.__music_name

    def set_music_name(self, music_name):
        self.__music_name = music_name

    def play(self):
        start_music(self.__music_name)

    @staticmethod
    def stop():
        stop_music()

    @staticmethod
    def query():
        get_music_status()


class ApolloMoveBase:
    def __init__(self):
        pass

    @staticmethod
    def get_move_status():
        try:
            print('------------------------------------')
            print('运动中.....')
            move_stat = get_robot_move_status()
            while move_stat['status'] != 0:
                if move_stat['status'] == 505:
                    print('运动程序不存在')
                    return False
                move_stat = get_robot_move_status()
            print('------------------------------------')
            print('运动完成.....')
            print('------------------------------------')
            return True
        except Exception as err:
            print('发生未知错误:{}'.format(err))
            return False

    @abstractmethod
    def move(self, distance):
        pass


class ApolloMoveForward(ApolloMoveBase):
    def __init__(self):
        super().__init__()

    def move(self, distance):
        robot_move_forward(distance)
        return self.get_move_status()


class ApolloMoveBack(ApolloMoveBase):
    def __init__(self):
        super().__init__()

    def move(self, distance):
        robot_move_back(distance)
        return self.get_move_status()


class ApolloRotateBase:
    def __init__(self):
        pass

    @staticmethod
    def get_rotate_status():
        try:
            print('------------------------------------')
            print('转动中.....')
            rotate_stat = get_robot_rotate_status()
            while rotate_stat['status'] != 0:
                if rotate_stat['status'] == 505:
                    print('运动程序不存在')
                    return False
                rotate_stat = get_robot_rotate_status()
            print('------------------------------------')
            print('转动完成.....')
            print('------------------------------------')
            return True
        except Exception as err:
            print('发生未知错误:{}'.format(err))
            return False

    @abstractmethod
    def rotate(self, angle):
        pass


class ApolloRotateRight(ApolloRotateBase):
    def __init__(self):
        super().__init__()

    def rotate(self, angle):
        robot_rotate_right(angle)
        return self.get_rotate_status()


class ApolloRotateLeft(ApolloRotateBase):
    def __init__(self):
        super().__init__()

    def rotate(self, angle):
        robot_rotate_left(angle)
        return self.get_rotate_status()


def robot_move_rectangle_sample():
    """
    自定义路线，让APOLLO小车走一个正方形，完成运动后播放歌曲
    向前走0.5m，到达后向右转90度，然后向前走0.5m，到达后向右转90度，接着再向前走0.5m，紧接着右转90度，
    最后再向前走0.5m到达起点位置，完成运动后播放歌曲
    :return:
    """
    try:
        # 初始化Apollo运动模块实例
        apollo_move_forward = ApolloMoveForward()
        # 初始化Apollo转动模块实例
        apollo_rotate_right = ApolloRotateRight()
        # 向前走0.5m
        print('向前运动0.5m')
        apollo_move_forward.move(0.5)
        # 向右转90度
        print('向右转动90度')
        apollo_rotate_right.rotate(90)
        # 向前走0.5m
        print('向前运动0.5m')
        apollo_move_forward.move(0.5)
        # 向右转90度
        print('向右转动90度')
        apollo_rotate_right.rotate(90)
        # 向前走0.5m
        print('向前运动0.5m')
        apollo_move_forward.move(0.5)
        # 向右转90度
        print('向右转动90度')
        apollo_rotate_right.rotate(90)
        # 向前走0.5m
        print('向前运动0.5m')
        apollo_move_forward.move(0.5)
        # 向右转90度
        print('向右转动90度')
        apollo_rotate_right.rotate(90)
    except Exception as err:
        print('运动模块发生未知错误:{}'.format(err))
        return
    try:
        # 初始化Apollo音乐模块实例
        apollo_music = ApolloMusicPlayer()
        # 播放音乐：来自天堂的魔鬼
        apollo_music.set_music_name('来自天堂的魔鬼')
        print('播放音乐:{}'.format('来自天堂的魔鬼'))
        apollo_music.play()
        # 暂停40s播放音乐
        time.sleep(40)
        # 关闭音乐播放
        apollo_music.stop()
    except Exception as err:
        print('音乐模块发生未知错误:{}'.format(err))
        return


if __name__ == '__main__':
    # 机器人规则运动
    robot_move_rectangle_sample()



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.voice import get_wakeup_status, open_asr, get_asr_status
from bayeslibs.voice import open_wakeup
from bayeslibs.voice import start_small_talk, start_speak, get_speak_status


def voice_play(text):
    """
       输入任意内容，让APOLLO机器人将其播放出来，实现语音合成功能
    """
    start_res = start_speak(text)
    if start_res and start_res['status'] == 0:
        res = get_speak_status()
        print('------------------------------------')
        print('小贝语音合成中.......')
        while res['status'] == 0:
            res = get_speak_status()
        return True
    else:
        return False


def apollo_chat(question):
    """
    语音输入自己想说的话，观察APOLLO的自然语言处理结果
    """
    result = start_small_talk(question)
    if result['status'] == 0:
        answer = result['answer']
        print('小贝的回答:{}'.format(answer))
    else:
        print('小贝还小，还有很多需要学习')
        answer = '小贝还小，还有很多需要学习'
    return answer


def auto_speech_recognize():
    """
    APOLLO机器人语音识别
    """
    res_asr = open_asr()
    if res_asr and res_asr['status'] == 0:
        stat = get_asr_status()
        print('------------------------------------')
        print('小贝识别中.......')
        while stat and stat['is_asr']:
            stat = get_asr_status()
        print('------------------------------------')
        if 'text' in stat and stat['text']:
            print('小贝识别成功:{}'.format(stat['text']))
            print('------------------------------------')
            res = stat['text']
            return True, res
        else:
            res = '不好意思，小贝没有听清你的声音'
            return False, res
    else:
        return False, '网络故障'


def wakeup():
    """
    唤醒小贝
    """
    open_res = open_wakeup()
    if open_res and open_res['status'] == 0:
        res = get_wakeup_status()
        print('------------------------------------')
        print('小贝等待唤醒中.......')
        while res['status'] != 0:
            res = get_wakeup_status()
        print('------------------------------------')
        print('小贝唤醒成功，唤醒角度:{}'.format(res['angle']))
        return True
    else:
        return False


def voice_chat_sample():
    """
    天气问询：唤醒 “小贝”，查询某一个城市（如常州）的实时天气，以及语音闲聊
    """
    # 首先进行语音唤醒
    w_res = wakeup()
    if not w_res:
        print('小贝唤醒失败，请监测网络设备')
        return False
    # 唤醒成功后进行语音识别
    ret, text_reg = auto_speech_recognize()
    if not ret:
        print('小贝识别失败，请重新唤醒和识别')
        return False
    answer = apollo_chat(text_reg)
    voice_play(answer)
    return True


if __name__ == '__main__':
    count_ = 0
    # 循环聊天10次，次数可修改
    while count_ < 10:
        voice_chat_sample()
        count_ += 1



# conding=utf-8
"""
@project:edubalibs
@language:Python3
@create:2019/6/28
@author:qianyang<qianyang@aibayes.com>
@description:none
"""
from bayeslibs.config import BayesLibsConfig

APOLLO_IP = '192.168.0.107'
APOLLO_PORT = 5000


def connect_apollo(ip=APOLLO_IP, port=APOLLO_PORT):
    """
    设置Apollo小车的IP和PORT
    :return:
    """
    bayeslibs_config = BayesLibsConfig(ip=ip, port=port)
    bayeslibs_config.connect()


if __name__ == '__main__':
    connect_apollo()
